using System;
using GrassValley.Mseries.AppServer;
using GrassValley.Mseries.AppServerLog;
using GrassValley.Mseries.ChanStatus;
using GrassValley.Mseries.ConfigMgr;
using GrassValley.Mseries.Control;
using GrassValley.Mseries.DVCapture;
using GrassValley.Mseries.Editor;
using GrassValley.Mseries.MediaMgr;
using GrassValley.Mseries.Security;
using GrassValley.Mseries.Status;
using GrassValley.Mseries.TransferQueue;
// Leave the following "css_reference" lines in when running with CS-Script. They're 
// more than just comments - they tell CS-Script which DLLs to load if the DLL 
// name does not match the namespace that it implements.
//css_reference Credentials.dll;
//css_reference ServerUtil.dll;

/// <summary>
/// This is a sample CS-Script app for controlling the Grass Valley K2 server. 
/// The code is regular C# code, but it's written for CS-Script which can
/// run it as a script from a DOS command prompt.
///
/// Go to the K2 AppServer Developer's Guide for more information:
/// http://www.gvgdevelopers.com/K2DevGuide/K2DevGuide.html#%5B%5BCS-Script%20Sample%20Scripts%5D%5D
///
/// For more information about CS-Script go to:
/// http://www.members.optusnet.com.au/~olegshilo/
/// </summary>
namespace K2Script
{
		
	class Test
	{
		static int Main(string[] args)
		{
			try
			{
				// get the application's name & create a unique suite name
				string appName = Environment.GetCommandLineArgs()[0]; 					
				string suiteName = appName + "_" + System.Guid.NewGuid().ToString("N");
			
				// fill in these values for your connection (i.e. hardcode them, pass in cmdline arguments, read from file, read from console, etc.)
				string host = "";
				string username = "";
				string password = "";
				string domain = "";
			
				// Connect to host & pass in credentials. The "using" block disposes connection when done.
				using ( Connection connection = new Connection(appName, suiteName, 
							host, username, password, domain) )
				{
					// do work only if we have a valid connection
					if ( connection.IsConnected )
					{
					
						//////////////////////////
						// DO WORK HERE
						//////////////////////////
						// create a media mgr
						using ( IMediaMgr mediaMgr = connection.AppServer.CreateMediaMgr(appName) )
						{
							// First, get a list of the clips by enumerating the assets in the 
							// "V:\default" bin. This will return a cookie that points the 
							// results. We can then query those results for the properties we want.
							int count = 0;
							int cookie = mediaMgr.EnumerateAssets("V:", "default", ref count);
					 
							// let's query for 3 properties of the clip: the name, when it was created, and the length
							string query = "name+created+lengthstr";
							
							// Other properties you can query:
							// "uri", "name", "assettype", "created", "modified", "attributes1", 
							// "attributes2", "videoformat", "videocompressiontype", "firsttimecodemask", 
							// "lengthstr", "maxlength", "markin", "markout", "markinstr", "markoutstr", 
							// "dropframe", "locked", "hidef"
							// See "http://www.gvgdevelopers.com/K2DevGuide/K2DevGuide.html" for other asset properties.

							// Let's only get info on the first 100 clips in the bin
							int maxCount = 100;
							object dataObj = null;

							// Now, let's query for the results we want: 
							// query it for the properties we setup above, 
							// start with the 0th element, 
							// go up to a max of 100 clips
							// capture the actual number of clips we're getting data back from,
							// and return the results in a data object
							mediaMgr.GetResultProperty( cookie, query, 0, maxCount, out count, out dataObj);
							
							Console.WriteLine("Found " + count + " clips in V:\\default\\");

							// GetResultProperty for clip data returns a 2 dimensional array
							// - the first dimension represents the clip
							// - the second dimension represents the properties of the clip
							
							// so let's cast the dataobj to a clip array first
							object[] clipArray = (object[]) dataObj;				
							
							// loop thru each clip
							foreach (object clip in clipArray)
							{
								// then cast the clip object to an array of properties for each clip
								object[] propertyArray = (object[]) clip;
									
								// and finally print out the clip properties for each clip
								Console.WriteLine("Clip: {0}  Created: {1}  Length: {2}", propertyArray[0], propertyArray[1], propertyArray[2]);
							}
					
							// make sure that you close the results after enumerating volumes, bins, or assets.
							// if you don't, you will run out of database selectors!
							mediaMgr.CloseResults(cookie);
							
						} // note: when using block goes out of scope the mediamgr is disposed
						
					}
				}
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
			}
			
			return 0;
		}
	}

	
	// Connection helper class: sets up credentials & hostname, connects to K2
	public class Connection: IDisposable
	{
		private bool _connected = false;
		private AppServerMgrProxy _appServerMgrProxy = null;
		private IAppServer _appServer = null;
	
		/// <summary>
		/// Object that connects to the specified K2 credentials
		/// </summary>
		/// <param name="appname">The application's name.</param>
		/// <param name="suitename">The suite's name.</param>
		/// <param name="host">The K2's host name.</param>
		/// <param name="username">The username.</param>
		/// <param name="password">The password.</param>
		/// <param name="domain">The domain.</param>
		public Connection(string appname, string suitename, string host, string username, string password, string domain)
		{
			try
			{
				//////////////////////////
				// SETUP
				//////////////////////////
				if ( null == host || host.Length == 0)
				{
					Console.WriteLine("Empty host name passed to Connection object.");
					return;
				}
			
					

				//////////////////////////
				// CONNECT TO K2
				//////////////////////////
				
				// create an AppServerMgr proxy object
				_appServerMgrProxy = new AppServerMgrProxy();
		
				// tell it which K2 we're connecting to
				_appServerMgrProxy.SetHost(host);
		
				// give it the user credentials we want to use
				if ( username.Length > 0 )
					_appServerMgrProxy.SetUserCredentials(username, password, domain, false);
		
				// connect to the K2's AppServerMgr
				if ( !_appServerMgrProxy.Connect() )
				{
					// if the connection failed, report an error and exit
					Console.WriteLine("ERROR: Could not connect to AppService on host '" + host + "'. Check the host name.");
					_appServerMgrProxy = null;
					return;
				}

				// if we got here we're connected to K2 AppService
				
				//////////////////////////
				// CREATE AN APPSERVER
				//////////////////////////
				// now create an AppServer
				bool newConnection = false;
				_appServer = _appServerMgrProxy.CreateAppServer(suitename, 
					appname, out newConnection);
					
				_connected = true;
				
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
			}
		}

		/// <summary>
		/// Gets a value indicating whether this instance is connected.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is connected; otherwise, <c>false</c>.
		/// </value>
		public bool IsConnected
		{
			get
			{
				return _connected;
			}
		}
		
		/// <summary>
		/// Gets the AppServerMgr.
		/// </summary>
		/// <value>The AppServerMgr</value>
		public AppServerMgrProxy AppServerMgr
		{
			get
			{
				if ( _appServerMgrProxy == null )
					Console.WriteLine("WARNING: AppServerMgrProxy object == null!");
				return _appServerMgrProxy;
			}
		}
		
		/// <summary>
		/// Gets the AppServer
		/// </summary>
		/// <value>The AppServer</value>
		public IAppServer AppServer
		{
			get
			{
				if ( _appServerMgrProxy == null )
					Console.WriteLine("WARNING: AppServerProxy object == null!");
				return _appServer;
			}
		}
		
		/// <summary>
		/// Disposes this instance.
		/// </summary>
		public void Dispose()
		{
			if ( _appServer != null )
				_appServer.CloseConnection();
		}
		
	}
}